Attribute VB_Name = "API_Windows"
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' API declarations for windows manipulation
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long) As Long
Public Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Declare Function GetWindow Lib "user32" (ByVal hwnd As Long, ByVal wCmd As Long) As Long
Declare Function GetDesktopWindow Lib "user32" () As Long
Declare Function GetWindowText Lib "user32" Alias "GetWindowTextA" (ByVal hwnd As Long, ByVal lpString As String, ByVal cch As Long) As Long
Public Declare Function LockWindowUpdate Lib "user32" (ByVal hwndLock As Long) As Long
Public Declare Function SetParent Lib "user32" (ByVal hWndChild As Long, ByVal hWndNewParent As Long) As Long
Public Declare Function GetParent Lib "user32" (ByVal hwnd As Long) As Long
Public Declare Function SetWindowPos Lib "user32" (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, ByVal x As Long, _
    ByVal y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
Public Declare Function MoveWindow Lib "user32" (ByVal hwnd As Long, ByVal x As Long, ByVal y As Long, _
    ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long) As Long
Declare Function ShowWindow& Lib "user32" (ByVal hwnd As Long, ByVal nCmdShow As Long)
Private Declare Function InvalidateRect Lib "user32" ( _
  ByVal hwnd As Long, lpRect As Any, _
  ByVal bErase As Long) As Long

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' Other API declarations
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Declare Sub OutputDebugString Lib "kernel32" Alias _
    "OutputDebugStringA" (ByVal lpOutputString As String)

Private Const GWL_STYLE = (-16)

Enum SWP_Flags
  SWP_NOSIZE = &H1
  SWP_NOMOVE = &H2
  SWP_NOZORDER = &H4
  SWP_NOREDRAW = &H8
  SWP_NOACTIVATE = &H10
  SWP_FRAMECHANGED = &H20      ' The frame changed: send WM_NCCALCSIZE
  SWP_SHOWWINDOW = &H40
  SWP_HIDEWINDOW = &H80
  SWP_NOCOPYBITS = &H100
  SWP_NOOWNERZORDER = &H200    ' Don't do owner Z ordering
  SWP_NOSENDCHANGING = &H400   ' Don't send WM_WINDOWPOSCHANGING
  
  SWP_DRAWFRAME = SWP_FRAMECHANGED
  SWP_NOREPOSITION = SWP_NOOWNERZORDER
  
  SWP_DEFERERASE = &H2000
  SWP_ASYNCWINDOWPOS = &H4000
End Enum

Public Enum SW_cmds
  SW_HIDE = 0
  SW_NORMAL = 1
  SW_SHOWNORMAL = 1
  SW_SHOWMINIMIZED = 2
  SW_MAXIMIZE = 3
  SW_SHOWMAXIMIZED = 3
  SW_SHOWNOACTIVATE = 4
  SW_SHOW = 5
  SW_MINIMIZE = 6
  SW_SHOWMINNOACTIVE = 7
  SW_SHOWNA = 8
  SW_RESTORE = 9
  SW_MAX = 10
  SW_SHOWDEFAULT = 10
End Enum

Public Enum WinMsgs
  WM_NULL = &H0
  WM_CREATE = &H1
  WM_DESTROY = &H2
  WM_MOVE = &H3
  WM_SIZE = &H5
  WM_ACTIVATE = &H6
  WM_SETFOCUS = &H7
  WM_KILLFOCUS = &H8
  WM_ENABLE = &HA
  WM_SETREDRAW = &HB
  WM_PAINT = &HF
  WM_ERASEBKGND = &H14
  WM_SYSCOLORCHANGE = &H15
  WM_WININICHANGE = &H1A
  WM_SETCURSOR = &H20
  WM_NEXTDLGCTL = &H28
  WM_DRAWITEM = &H2B
  WM_MEASUREITEM = &H2C

  WM_SETFONT = &H30
  WM_GETFONT = &H31
  WM_WINDOWPOSCHANGED = &H47
  WM_NOTIFY = &H4E
  WM_NCCREATE = &H81
  WM_NCDESTROY = &H82
  WM_NCCALCSIZE = &H83
  WM_GETDLGCODE = &H87
  WM_KEYDOWN = &H100
  WM_KEYUP = &H101
  WM_CHAR = &H102
  WM_COMMAND = &H111
  WM_TIMER = &H113
  WM_HSCROLL = &H114
  WM_VSCROLL = &H115
  WM_INITMENUPOPUP = &H117

  WM_MOUSEMOVE = &H200
  WM_LBUTTONDOWN = &H201
  WM_LBUTTONUP = &H202
  WM_LBUTTONDBLCLK = &H203
  WM_RBUTTONDOWN = &H204
  WM_RBUTTONUP = &H205
  WM_RBUTTONDBLCLK = &H206
  WM_MBUTTONDOWN = &H207
  WM_MBUTTONUP = &H208
  WM_MBUTTONDBLCLK = &H209

  WM_USER = &H400
End Enum   ' WinMsgs

Public Enum WinStyles
  WS_OVERLAPPED = &H0
  WS_TABSTOP = &H10000
  WS_MAXIMIZEBOX = &H10000
  WS_MINIMIZEBOX = &H20000
  WS_GROUP = &H20000
  WS_THICKFRAME = &H40000
  WS_SYSMENU = &H80000
  WS_HSCROLL = &H100000
  WS_VSCROLL = &H200000
  WS_DLGFRAME = &H400000
  WS_BORDER = &H800000
  WS_CAPTION = (WS_BORDER Or WS_DLGFRAME)
  WS_MAXIMIZE = &H1000000
  WS_CLIPCHILDREN = &H2000000
  WS_CLIPSIBLINGS = &H4000000
  WS_DISABLED = &H8000000
  WS_VISIBLE = &H10000000
  WS_MINIMIZE = &H20000000
  WS_CHILD = &H40000000
  WS_POPUP = &H80000000

  WS_TILED = WS_OVERLAPPED
  WS_ICONIC = WS_MINIMIZE
  WS_SIZEBOX = WS_THICKFRAME

  ' Common Window Styles
  WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
  WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
  WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU)
  WS_CHILDWINDOW = WS_CHILD
End Enum   ' WinStyles

Public Enum WinStylesEx
  WS_EX_DLGMODALFRAME = &H1
  WS_EX_NOPARENTNOTIFY = &H4
  WS_EX_TOPMOST = &H8
  WS_EX_ACCEPTFILES = &H10
  WS_EX_TRANSPARENT = &H20

  WS_EX_MDICHILD = &H40
  WS_EX_TOOLWINDOW = &H80
  WS_EX_WINDOWEDGE = &H100
  WS_EX_CLIENTEDGE = &H200
  WS_EX_CONTEXTHELP = &H400

  WS_EX_RIGHT = &H1000
  WS_EX_LEFT = &H0
  WS_EX_RTLREADING = &H2000
  WS_EX_LTRREADING = &H0
  WS_EX_LEFTSCROLLBAR = &H4000
  WS_EX_RIGHTSCROLLBAR = &H0

  WS_EX_CONTROLPARENT = &H10000
  WS_EX_STATICEDGE = &H20000
  WS_EX_APPWINDOW = &H40000

  WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_CLIENTEDGE)
  WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_TOOLWINDOW Or WS_EX_TOPMOST)
End Enum   ' WinStylesEx

'Reparent a given window to act as a child of another window
Public Sub ChangeParentWindow(ByVal hWndChild As Long, ByVal hwndParent As Long, ByVal bPopupStyle As Boolean)
    Dim dwStyle As Long
    If hWndChild = 0 Or hwndParent = 0 Then Exit Sub
    'Get the old style...
    dwStyle = GetWindowLong(hWndChild, GWL_STYLE)
    'Set the WS_CHILD style bit...
    dwStyle = dwStyle Or WS_CHILD Or WS_POPUP
    'Set the new style...
    SetWindowLong hWndChild, GWL_STYLE, dwStyle
    'Set the parent...
    SetParent hWndChild, hwndParent
    'Clear the WS_POPUP style bit...
    If Not bPopupStyle Then SetWindowLong hWndChild, GWL_STYLE, dwStyle And Not WS_POPUP
    'Now hWndChild is parented and movement is relative to the
    'parent control, i.e., coordinate 0,0 is the upper left corner
    'of hWndParent's client area.
End Sub

'Invalidate and redraw all the client area
Public Sub RedrawAll(ByVal hwnd As Long)
    InvalidateRect hwnd, ByVal 0&, 1
    SetWindowPos hwnd, 0&, 0&, 0&, 0&, 0&, _
        SWP_FRAMECHANGED Or SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOZORDER
End Sub

'Emit a debug message
'Use DbgView from www.sysinternals.com to trace debug informations
Public Sub DBG(ByVal sDebugMsg As String)
    Debug.Print sDebugMsg
    OutputDebugString sDebugMsg
End Sub

