VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Commands"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public m_SEapp As SolidEdgeFramework.Application
Attribute m_SEapp.VB_VarHelpID = -1
Public WithEvents m_addinEvents As SolidEdgeFramework.AddInEvents
Attribute m_addinEvents.VB_VarHelpID = -1
Public WithEvents m_AppEvents As SolidEdgeFramework.ApplicationEvents
Attribute m_AppEvents.VB_VarHelpID = -1
Public m_myAddIn As SolidEdgeFramework.AddIn
Attribute m_myAddIn.VB_VarHelpID = -1

'Every command button have it's own ID assigned by Solid Edge upon startup
'Public m_Button1_ID As Long
'Public m_Button2_ID As Long
'Public m_Button3_ID As Long

'Collection of cEdgeBar objects
Public cEdgeBars As Collection

Private Sub Class_Initialize()
    'Initialize the edgebar form collection
    Set cEdgeBars = New Collection
End Sub

Private Sub Class_Terminate()
    Dim fBar As frmBar
    'Ensure the edgebar collection is fully unloaded
    For Each fBar In cEdgeBars
        Unload fBar
    Next
    Set fBar = Nothing
    Set cEdgeBars = Nothing
End Sub

' NOTE: If a command bar button has been added to the GUI via the AddCommandBarButton,
' ----  as the Addin is being connected up, then essentially, it behaves slightly
'       different from the non-addin command bar buttons added via automation. The
'       latter fire the CommandBarButtonEvents, but the former only fire AddInEvents

Private Sub m_addinEvents_OnCommand(ByVal nCmdID As Long)
    
'    Select Case nCmdID
'        Case CmdId_Button1
'            MsgBox "You have just pressed the button 1.", vbInformation
'        Case CmdId_Button2
'            MsgBox "You have just pressed the button 2.", vbInformation
'        Case CmdId_Button3
'            MsgBox "You have just pressed the button 3.", vbInformation
'        Case Else
'            MsgBox "Unrecognized CommandId: " & nCmdID
'    End Select
End Sub

Private Sub m_addinEvents_OnCommandHelp(ByVal hFrameWnd As Long, ByVal uHelpCommand As Long, ByVal nCmdID As Long)
    'MsgBox "OnCommandHelp event"
End Sub

Private Sub m_addinEvents_OnCommandUpdateUI(ByVal nCmdID As Long, lCmdFlags As Long, MenuItemText As String, nIDBitmap As Long)
    'MsgBox "OnCommandUpdateUI event " & nCmdID
End Sub

''''''''
''''''''''''''''''''''''''''''''''''''''''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Sub m_AppEvents_BeforeDocumentSave(ByVal theDocument As Object)
    If m_SEapp Is Nothing Then Exit Sub
    'I don't want my addin to trigger a document save when it is
    'hidden because it may interfere with a remote automation client
    If Not m_SEapp.Visible Then Exit Sub
    DBG "Addin: BeforeDocumentSave " & GetSafeDocName(theDocument)
End Sub

Private Sub m_AppEvents_AfterActiveDocumentChange(ByVal theDocument As Object)
    'This event may be ignored because the switching of EdgeBar is
    'handled automatically by Solid Edge
    DBG "AddIn: AfterActiveDocumentChange " & GetSafeDocName(theDocument)
End Sub

Private Sub m_AppEvents_AfterDocumentOpen(ByVal theDocument As Object)
    On Error GoTo fail
    DBG "AddIn: AfterDocumentOpen " & GetSafeDocName(theDocument)
    'Add an edgebar for this document
10  AddEdgeBarForDocument theDocument
    Exit Sub
fail:
    ErrHandler "AfterDocumentOpen" & GetSafeDocName(theDocument)
End Sub

Private Sub m_AppEvents_AfterNewDocumentOpen(ByVal theDocument As Object)
    On Error GoTo fail
    DBG "AddIn: AfterNewDocumentOpen " & GetSafeDocName(theDocument)
    'Add an edgebar for this new document
10  AddEdgeBarForDocument theDocument
    Exit Sub
fail:
    ErrHandler "AfterNewDocumentOpen" & GetSafeDocName(theDocument)
End Sub

Private Sub m_AppEvents_BeforeDocumentClose(ByVal theDocument As Object)
    On Error GoTo fail
    DBG "AddIn: BeforeDocumentClose " & GetSafeDocName(theDocument)
    'Remove the edgebar for this document while closing
10  RemoveEdgeBarForDocument theDocument
    Exit Sub
fail:
    ErrHandler "AfterNewDocumentOpen" & GetSafeDocName(theDocument)
End Sub

' Given a single document activate the edgebar for that document
 Sub AddEdgeBarForDocument(theDocument As Object)
    Dim hEdgeBarWnd As Long
    Dim fBar As frmBar
    
    On Error GoTo fail
    'Assert that an EdgeBar has not been already created for this document
    If theDocument Is Nothing Then Exit Sub
10  If IsInCollection(cEdgeBars, theDocument.FullName) Then Exit Sub
        
    'If not found in collection create a new EdgeBarPage for this document
20  hEdgeBarWnd = AddEdgeBarPage(theDocument)
    If hEdgeBarWnd = 0 Then Exit Sub
        
    'Create a new frmBar instance
30  Set fBar = New frmBar
    Load fBar
    
    'The HWND of the EdgeBar is stored inside the form
    'for support the RemovePage() method that requires
    'the same exact HWND for EdgeBar removal
40  fBar.hEdgeBarWnd = hEdgeBarWnd
    
    'Add the EdgeBar form to collection
50  cEdgeBars.Add fBar, theDocument.FullName
    
    'Reparent the form to the EdgeBar window provided by solid edge
60  ChangeParentWindow fBar.hwnd, fBar.hEdgeBarWnd, False
    DBG "AddIn: ChangeParentWindow " & fBar.hEdgeBarWnd
    
    'NOTE: ActiveX DLL cant show non-modal forms, so must
    'call ShowWindow directly to overcome this limitation.
    'The maximize style is required to make the VB form
    'to resize to the exact dimension of the EdgeBar pane
70  ShowWindow fBar.hwnd, SW_cmds.SW_SHOWMAXIMIZED
    
    'Init the form with a reference to the active document
80  fBar.InitForm m_SEapp, theDocument
    
    Exit Sub
fail:
    ErrHandlerRaise "AddEdgeBarForDocument"
End Sub

' Given a single document remove the edgebar for that document
Private Sub RemoveEdgeBarForDocument(theDocument As Object)
    Dim fBar As frmBar
    
    On Error GoTo fail
10  If theDocument Is Nothing Then Exit Sub
    
    'Make sure to remove only previously instantiated forms
20  If Not IsInCollection(cEdgeBars, theDocument.FullName) Then Exit Sub

    'Destroy the current frmBar instance and remove it from collection
30  Set fBar = cEdgeBars(theDocument.FullName)
40  RemoveEdgeBarPage fBar.hEdgeBarWnd, theDocument
50  cEdgeBars.Remove theDocument.FullName
60  Unload fBar
    
    'The form should be terminated to avoid memory leakage
70  Set fBar = Nothing
    Exit Sub
fail:
    ErrHandlerRaise "RemoveEdgeBarForDocument"
End Sub

' Wrapper for create a new EdgeBar frame, the HWND is returned
Private Function AddEdgeBarPage(theDocument As Object) As Long
    Dim oSEB As SolidEdgeFramework.ISolidEdgeBar
    Dim hEdgeBarWnd As Long
    
    On Error GoTo fail
10  Set oSEB = m_myAddIn
20  hEdgeBarWnd = oSEB.AddPage(theDocument, App.hInstance, 105, "VB Addin SDK", 0)
    If hEdgeBarWnd = 0 Then Exit Function
    AddEdgeBarPage = hEdgeBarWnd
    DBG "AddIn: AddEdgeBarPage " & hEdgeBarWnd
    Exit Function
fail:
    ErrHandlerRaise "AddEdgeBarPage"
End Function

' Wrapper to remove an existing EdgeBar frame
Private Sub RemoveEdgeBarPage(ByVal hEdgeBarWnd As Long, theDocument As Object)
    Dim oSEB As SolidEdgeFramework.ISolidEdgeBar
    On Error GoTo fail
10  Set oSEB = m_myAddIn
20  oSEB.RemovePage theDocument, hEdgeBarWnd, 0&
    DBG "AddIn: RemoveEdgeBarPage: " & hEdgeBarWnd
    Exit Sub
fail:
    ErrHandlerRaise "RemoveEdgeBarPage"
End Sub

' Wrapper to activate an existing EdgeBar frame
Private Sub ActivateEdgeBarPage(ByVal hEdgeBarWnd As Long, theDocument As Object)
    Dim oSEB As SolidEdgeFramework.ISolidEdgeBar
    On Error GoTo fail
20  Set oSEB = m_myAddIn
30  oSEB.SetActivePage theDocument, hEdgeBarWnd, 0&
    DBG "AddIn: ActivateEdgeBarPage: " & hEdgeBarWnd
    Exit Sub
fail:
    ErrHandlerRaise "ActivateEdgeBarPage"
End Sub

